@@ -4,6 +4,8 @@ import lark
 
 import logging
 
+from ..physics import element
+
 logger = logging.getLogger(__name__)
 
 grammar = '''\
@@ -198,7 +200,7 @@ TTLS_GNAME_2_22: "Other adatom redispl thresh"
 '''
 
 grammar_ttls_hname = '''\
-TTLS_HNAME: TTLS_HNAME_1 | TTLS_HNAME_3 | TTLS_HNAME_4 | TTLS_HNAME_5 
+TTLS_HNAME: TTLS_HNAME_1 | TTLS_HNAME_3 | TTLS_HNAME_4 | TTLS_HNAME_5
 TTLS_HNAME_1TO4: TTLS_HNAME_1 | TTLS_HNAME_3 | TTLS_HNAME_4
 TTLS_HNAME_1: ".....Number of....."
 // TTLS_HNAME_2: ".....Number of....."
@@ -388,22 +390,31 @@ TTLS_ZNAME_7: "atoms sputtered from side surfaces: energy spectrum"
 TTLS_ZNAME_8: "atoms sputtered from other surface: energy spectrum"
 '''
 
-grammar_elem = '''\
-ELEM: "H" | "He"
-    | "Li" | "Be" | "B" | "C" | "N" | "O" | "F" | "Ne"
-    | "Na" | "Mg" | "Al" | "Si" | "P" | "S" | "Cl" | "Ar"
-    | "K" | "Ca" | "Sc" | "Ti" | "V" | "Cr" | "Mn" | "Fe" | "Co"
-    | "Ni" | "Cu" | "Zn" | "Ga" | "Ge" | "As" | "Se" | "Br" | "Kr"
-    | "Rb" | "Sr" | "Y" | "Zr" | "Nb" | "Mo" | "Tc" | "Ru" | "Rh"
-    | "Pd" | "Ag" | "Cd" | "In" | "Sn" | "Sb" | "Te" | "I" | "Xe"
-    | "Cs" | "Ba" | "La" | "Ce" | "Pr" | "Nd" | "Pm" | "Sm" | "Eu"
-    | "Gd" | "Tb" | "Dy" | "Ho" | "Er" | "Tm" | "Yb" | "Lu" | "Hf"
-    | "Ta" | "W" | "Re" | "Os" | "Ir" | "Pt" | "Au" | "Hg" | "Tl"
-    | "Pb" | "Bi" | "Po" | "At" | "Rn" | "Fr" | "Ra" | "Ac" | "Th"
-    | "Pa" | "U" | "Np" | "Pu" | "Am" | "Cm" | "Bk" | "Cf" | "Es"
-    | "Fm" | "Md" | "No" | "Lr" | "Rf" | "Db" | "Sg" | "Bh" | "Hs"
-    | "Mt" | "Ds" | "Rg" | "Cn" | "Uut" | "Fl" | "Uup" | "Lv" | "Uus" | "Uuo"
-'''
+# create ELEM terminal
+# terminal elements should be sorted by the length,
+# so that the parser can find the longer element name (ex. Si) faster than the shorter one (ex. S)
+#
+# grammar_elem_wrong = '''\
+# ELEM: "H" | "He"
+#     | "Li" | "Be" | "B" | "C" | "N" | "O" | "F" | "Ne"
+#     | "Na" | "Mg" | "Al" | "Si" | "P" | "S" | "Cl" | "Ar"
+#     | "K" | "Ca" | "Sc" | "Ti" | "V" | "Cr" | "Mn" | "Fe" | "Co"
+#     | "Ni" | "Cu" | "Zn" | "Ga" | "Ge" | "As" | "Se" | "Br" | "Kr"
+#     | "Rb" | "Sr" | "Y" | "Zr" | "Nb" | "Mo" | "Tc" | "Ru" | "Rh"
+#     | "Pd" | "Ag" | "Cd" | "In" | "Sn" | "Sb" | "Te" | "I" | "Xe"
+#     | "Cs" | "Ba" | "La" | "Ce" | "Pr" | "Nd" | "Pm" | "Sm" | "Eu"
+#     | "Gd" | "Tb" | "Dy" | "Ho" | "Er" | "Tm" | "Yb" | "Lu" | "Hf"
+#     | "Ta" | "W" | "Re" | "Os" | "Ir" | "Pt" | "Au" | "Hg" | "Tl"
+#     | "Pb" | "Bi" | "Po" | "At" | "Rn" | "Fr" | "Ra" | "Ac" | "Th"
+#     | "Pa" | "U" | "Np" | "Pu" | "Am" | "Cm" | "Bk" | "Cf" | "Es"
+#     | "Fm" | "Md" | "No" | "Lr" | "Rf" | "Db" | "Sg" | "Bh" | "Hs"
+#     | "Mt" | "Ds" | "Rg" | "Cn" | "Uut" | "Fl" | "Uup" | "Lv" | "Uus" | "Uuo"
+# '''
+
+elem_symbols = list(element.table_bysym.keys())
+elem_symbols.remove('None')
+grammar_elem = 'ELEM: ' + '|'.join([f'"{a}"' for a in sorted(elem_symbols, key=lambda s: (-len(s), s))])
+
 
 class Transformer(lark.Transformer):
     def int(self, args):
@@ -412,6 +423,7 @@ class Transformer(lark.Transformer):
     def float(self, args):
         return float(args[0])
 
+
 # add fwint{width} recievers
 def gen_fixedwidth_int_reciever(width):
     def func(self, args):
